/*------------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        18.12.2011
 Description:    DMX_Switch
------------------------------------------------------------------------------*/

#include <avr/interrupt.h>
#include <avr/io.h>
#include <stdio.h>

#define F_CPU 12000000

#define DMX_BAUD 250000
#define DMX_LOST_TIMEOUT 500

volatile unsigned char dmx_buffer[4] = {0,0,0,0};
volatile unsigned int dmx_lost = DMX_LOST_TIMEOUT;

volatile unsigned int dmx_adresse = 0;

//############################################################################
//DMX Senderoutine
ISR (USART_RX_vect)
//############################################################################
{
	static unsigned int dmx_channel_rx_count = 0;
	static unsigned char dmx_valid = 0;
	unsigned char tmp = 0;
	
	tmp =  UDR0;
	
	if(UCSR0A&(1<<FE0))
	{
		if(dmx_channel_rx_count > 1)
		{
			dmx_lost = 0;
		}
		dmx_channel_rx_count = 0;	
		dmx_buffer[0] = tmp;
		if(dmx_buffer[0] == 0)
		{
			dmx_valid = 1;
			dmx_channel_rx_count++;
		}
		else
		{
			dmx_valid = 0;
		}
		return;
	}
	
	if(dmx_valid)
	{
		if(dmx_channel_rx_count == dmx_adresse) dmx_buffer[1] = tmp;
		if(dmx_channel_rx_count == dmx_adresse+1) dmx_buffer[2] = tmp;
		
		if(dmx_channel_rx_count < 514)
		{
			dmx_channel_rx_count++;
		}
		return;
	}
}

//############################################################################
//Hier wird die Zeit gezhlt (Tick 1ms)
ISR (TIMER2_COMPA_vect)
//############################################################################
{
	if(dmx_lost<DMX_LOST_TIMEOUT)
	{
		dmx_lost++;
	}
}

//############################################################################
//Hauptprogramm
int main (void) 
//############################################################################
{  
	unsigned int dmx_adresse_tmp;
	
	//Init usart DMX-BUS
	UBRR0   = (F_CPU / (DMX_BAUD * 16L) - 1);
	UCSR0B|=(1 << RXEN0 | 1<< RXCIE0);
	UCSR0C|=(1<<USBS0); //USBS0 2 Stop bits	
	sei();//Globale Interrupts Enable
	
	//Timer2 Timecounter fr DMX Ausfall
	TCCR2A |= (1<<WGM21);
	TCCR2B |= (1<<CS22|1<<CS21|1<<CS20);
	TIMSK2 |= (1<<OCIE2A);
	OCR2A = F_CPU/1024/1000 - 1; //Tick 1ms
		
	DDRD |= (1<<PD7); //Output
	DDRD |= (1<<PD6); //Output
	
	PORTB |= 0x3F;
	PORTC |= 0xFF;
	
	//Endlosschleife
	while(1)
	{
		dmx_adresse_tmp = 0;
		
		if(!(PINB&(1<<PB0))) dmx_adresse_tmp |= 0x01;
 		if(!(PINB&(1<<PB1))) dmx_adresse_tmp |= 0x02;
		if(!(PINB&(1<<PB2))) dmx_adresse_tmp |= 0x04;
		if(!(PINB&(1<<PB3))) dmx_adresse_tmp |= 0x08;
		if(!(PINB&(1<<PB4))) dmx_adresse_tmp |= 0x10;
		if(!(PINB&(1<<PB5))) dmx_adresse_tmp |= 0x20;
		
		if(!(PINC&(1<<PC0))) dmx_adresse_tmp |= 0x40;
		if(!(PINC&(1<<PC1))) dmx_adresse_tmp |= 0x80;
		if(!(PINC&(1<<PC2))) dmx_adresse_tmp |= 0x0100;
		
		if(dmx_adresse != dmx_adresse_tmp) dmx_adresse =  dmx_adresse_tmp;
		
		
		if(dmx_buffer[1] > 127)
		{
			PORTD |= (1<<PD7); //1. Kanal ON
		}
		else
		{
			PORTD &=~(1<<PD7); //1. Kanal OFF
		}
		
		
		if(dmx_buffer[2] > 127)
		{
			PORTD |= (1<<PD6); //2. Kanal ON
		}
		else
		{
			PORTD &=~(1<<PD6); //2. Kanal OFF
		}
		
		
		if(dmx_lost==DMX_LOST_TIMEOUT)
		{
			dmx_buffer[1] = 0;
			dmx_buffer[2] = 0;
		}
	}
}